# Copyright 2009 Canonical Ltd.
#
# This file is part of desktopcouch.
#
#  desktopcouch is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# desktopcouch is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with desktopcouch.  If not, see <http://www.gnu.org/licenses/>.
"Desktop Couch helper files"

from __future__ import with_statement
import os
import re
import errno
import time

def find_pid(start_if_not_running=True):
    # Work out whether CouchDB is running by looking at its pid file
    def get_pid():
        from desktopcouch import local_files
        try:
            pid_file = local_files.FILE_PID
            with open(pid_file) as fp:
                try:
                    return int(fp.read())
                except ValueError:
                    return None
        except IOError:
            return None

    pid = get_pid()
    if not process_is_couchdb(pid) and start_if_not_running:
        # pidfile is stale
        # start CouchDB by running the startup script
        print "Desktop CouchDB is not running; starting it.",
        from desktopcouch import start_local_couchdb
        start_local_couchdb.start_couchdb()
        for timeout in xrange(1000):
            pid = get_pid()
            if process_is_couchdb(pid):
                break
            print ".",
            time.sleep(0.1)

        if process_is_couchdb(pid):
            print " done."
        else:
            print " failed."
            raise RuntimeError("desktop-couch not started")

    return pid

def process_is_couchdb__linux(pid):
    if not isinstance(pid, int):
        return False

    proc_dir = "/proc/%s" % (pid,)

    try:
        # check to make sure it is actually a desktop-couch instance
        with open(os.path.join(proc_dir, 'cmdline')) as cmd_file:
            cmd = cmd_file.read()
        if '/desktop-couch' not in cmd:
            return False

        # make sure it's our process.
        if not os.access(os.path.join(proc_dir, "mem"), os.W_OK):
            return False

    except IOError:
        return False

    return True


def find_port__linux(pid):
    # Look in the CouchDB log to find the port number, someday.
    # Currently, we have to grovel around in /proc instead.
    # Oh, the huge manatee... (this replaced an lsof shell recipe
    # which was shorter but less reliable)

    proc_dir = "/proc/%s" % (pid,)
    
    # enumerate the process' file descriptors
    fd_dir = os.path.join(proc_dir, 'fd')
    try:
        fd_paths = [os.readlink(os.path.join(fd_dir, fd))
                    for fd in os.listdir(fd_dir)]
    except OSError:
        raise RuntimeError("Unable to find file descriptors in /proc")
        

    # identify socket fds
    socket_matches = [re.match('socket:\\[([0-9]+)\\]', p) for p in fd_paths]
    # extract their inode numbers
    socket_inodes = [m.group(1) for m in socket_matches if m is not None]

    # construct a subexpression which matches any one of these inodes
    inode_subexp = "|".join(socket_inodes)
    # construct regexp to match /proc/net/tcp entries which are listening
    # sockets having one of the given inode numbers
    listening_regexp = re.compile(r'''
        \s*\d+:\s*                # sl
        0100007F:([0-9A-F]{4})\s+ # local_address
        00000000:0000\s+          # rem_address
        0A\s+                     # st (0A = listening)
        [0-9A-F]{8}:              # tx_queue
        [0-9A-F]{8}\s+            # rx_queue
        [0-9A-F]{2}:              # tr
        [0-9A-F]{8}\s+            # tm->when
        [0-9A-F]{8}\s*            # retrnsmt
        \d+\s+\d+\s+              # uid, timeout
        (?:%s)\s+                 # inode
        ''' % (inode_subexp,), re.VERBOSE)

    # extract the TCP port from the first matching line in /proc/$pid/net/tcp
    port = None
    with open(os.path.join(proc_dir, 'net', 'tcp')) as tcp_file:
        for line in tcp_file:
            match = listening_regexp.match(line)
            if match is not None:
                port = str(int(match.group(1), 16))
                break
    if port is None:
        raise RuntimeError("Unable to find listening port")

    return port



import platform
os_name = platform.system()
try:
    process_is_couchdb = {
        "Linux":process_is_couchdb__linux
        } [os_name]

    find_port = {
        "Linux":find_port__linux
        } [os_name]
except KeyError:
    raise NotImplementedError("os %r is not yet supported" % (os_name,))
