# Copyright 2009 Canonical Ltd.
#
# This file is part of desktopcouch-contacts.
#
#  desktopcouch is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# desktopcouch is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with desktopcouch.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Stuart Langridge <stuart.langridge@canonical.com>
#          Nicola Larosa <nicola.larosa@canonical.com>

"""Creating CouchDb-stored contacts for testing"""

import random, string, uuid

from couchdb import Server

CONTACT_DOCTYPE = 'http://www.freedesktop.org/wiki/Specifications/desktopcouch/contact'
COUCHDB_SYS_PORT = 5984

FIRST_NAMES = ('Jack', 'Thomas', 'Oliver', 'Joshua', 'Harry', 'Charlie',
    'Daniel', 'William', 'James', 'Alfie', 'Grace', 'Ruby', 'Olivia',
    'Emily', 'Jessica', 'Sophie', 'Chloe', 'Lily', 'Ella', 'Amelia')
LAST_NAMES = ('Dalglish', 'Grobbelaar', 'Lawrenson', 'Beglin', 'Nicol',
    'Whelan', 'Hansen', 'Johnston', 'Rush', 'Molby', 'MacDonald', 'McMahon')
CITIES = ('Scunthorpe', 'Birmingham', 'Cambridge', 'Durham', 'Bedford')
COUNTRIES = ('England', 'Ireland', 'Scotland', 'Wales')

def random_string(length=10):
    """Return a string of random lowercase letters and of specified length"""
    return ''.join(random.sample(string.lowercase, length))

def random_postal_address():
    """Return something that looks like a postal address"""
    return '%s Street' % random.choice(LAST_NAMES)

def random_email_address(first_name, last_name):
    """Return something that looks like an email address"""
    return '%s.%s@%s.com' % (first_name, last_name, random_string(3))

def random_postal_code():
    """Return something that looks like a postal code"""
    return '%s12 3%s' % (random_string(2), random_string(2))

def random_phone_number():
    """Return something that looks like a phone number"""
    return '+%s %s %s %s' % (random.randint(10, 99), random.randint(10, 99),
      random.randint(1000, 9999), random.randint(1000, 9999))

def random_birth_date():
    """Return something that looks like a birth date"""
    return '%04d-%02d-%02d' % (random.randint(1900, 2006),
        random.randint(1, 12), random.randint(1, 28))

def make_one_contact(maincount, doctype, app_annots):
    """Make up one contact randomly"""
    # Record schema
    fielddict = {'record_type': doctype, 'record_type_version': '1.0'}
    # Names
    first_name = random.choice(FIRST_NAMES) + str(maincount)
    last_name = random.choice(LAST_NAMES)
    # Addresses
    addresses = {}
    for address_type in ('home', 'work', 'other'):
        addresses[str(uuid.uuid4())] = {
            'address1': random_postal_address(), 'address2': '',
            'pobox': '', 'city': random.choice(CITIES),
            'state': '', 'postalcode': random_postal_code(),
            'country': random.choice(COUNTRIES), 'description': address_type}
    # Email addresses
    email_addresses = {}
    for email_address_type in ('home', 'work', 'other'):
        email_addresses[str(uuid.uuid4())] = {
            'address': random_email_address(first_name, last_name),
            'description': email_address_type}
    # Phone numbers
    phone_numbers = {}
    for phone_number_type in ('home', 'work', 'other'):
        phone_numbers[str(uuid.uuid4())] = {
            'priority': 0, 'number': random_phone_number(),
            'description': phone_number_type}
    # Store data in fielddict
    fielddict.update({'first_name': first_name, 'last_name': last_name,
        'addresses': addresses, 'email_addresses': email_addresses,
        'phone_numbers': phone_numbers, 'birth_date': random_birth_date()})
    # Possibly add example application annotations
    if app_annots:
        fielddict['application_annotations'] = app_annots
    return fielddict

def create_many_contacts(
        num_contacts=10, host='localhost', port=COUCHDB_SYS_PORT,
        db_name='contacts', doctype=CONTACT_DOCTYPE, app_annots=None):
    """Make many contacts and create their records"""
    server_url = 'http://%s:%s/' % (host, port)
    server = Server(server_url)
    db = server[db_name] if db_name in server else server.create(db_name)
    record_ids = []
    for maincount in range(1, num_contacts + 1):
        # Make the contact
        fielddict = make_one_contact(maincount, doctype, app_annots)
        # Store data in CouchDB
        record_id = db.create(fielddict)
        record_ids.append(record_id)
    return record_ids
